#include "scoredialog.h"
#include "xpm/mark.h"

ScoreDialog::ScoreDialog(ScoreFile *scoreFile0,MineTimer *timer0){
  scoreFile=scoreFile0;
  timer=timer0;


  scrWin = new Gtk::ScrolledWindow;
  scoreCol = new ScoreCol();
  mList = Gtk::ListStore::create(*scoreCol);
  listView= new Gtk::TreeView(mList);
  listView->append_column("Best",scoreCol->num);
  listView->append_column("Score",scoreCol->score);
  listView->append_column("Board",scoreCol->strBoard);
  listView->append_column("Bomb",scoreCol->bomb);
  listView->append_column("Time",scoreCol->strSec);
  cellProg = new Gtk::CellRendererProgress;
  int cols_count=listView->append_column("Bomb Rate",*cellProg);
  progCol=listView->get_column(cols_count-1);
  if(progCol){
#ifdef GLIBMM_PROPERTIES_ENABLED
    progCol->add_attribute(cellProg->property_value(),scoreCol->rate);
#else
    progCol->add_attribute(*cellProg,"value",scoreCol->rate);
#endif 
  }
  listView->append_column("Name",scoreCol->strName);
  listView->append_column("Mode",scoreCol->strMode);
  
  (listView->get_column(0))->set_sort_column(scoreCol->num);
  (listView->get_column(1))->set_sort_column(scoreCol->score);
  (listView->get_column(2))->set_sort_column(scoreCol->board);
  (listView->get_column(3))->set_sort_column(scoreCol->bomb);
  (listView->get_column(4))->set_sort_column(scoreCol->sec);
  (listView->get_column(5))->set_sort_column(scoreCol->rate);
  
  beginnerCheck= new Gtk::CheckButton("Beginner",false);
  mediumCheck= new Gtk::CheckButton("Medium",false);
  advanceCheck= new Gtk::CheckButton("Advance",false);
  freeCheck= new Gtk::CheckButton("Free",false);

  checkBox= new Gtk::Table(4,1,false);
  
  vbox=get_vbox();

  okButton=add_button("OK",Gtk::RESPONSE_NONE);

  beginnerFlag=1;
  mediumFlag=1;
  advanceFlag=1;
  freeFlag=1;
  targetRow=0;

  set_title("SCORE");
  scoreIcon=Gdk::Pixbuf::create_from_xpm_data(flag_xpm);
  set_icon(scoreIcon);
  set_position(Gtk::WIN_POS_CENTER);
  set_keep_above(true);
}

ScoreDialog::~ScoreDialog(){
  ScoreItem *node;

  node=scoreFile->firstList;

  while(node){
    node->select=0;
    node=node->next;
  }
 
  mList->clear();
  delete okButton;
  delete beginnerCheck;
  delete mediumCheck;
  delete advanceCheck;
  delete freeCheck;
  delete checkBox;
  delete cellProg;
  delete scoreCol;
  delete listView;
  delete scrWin;
}

bool ScoreDialog::SignalDelete(GdkEventAny *event){
  std::cout<<"delete...\n";
   remove_modal_grab();
   
   if(timeGrab==0){
     timer->Start();
   }

  delete this;
  return 1;
}

void ScoreDialog::OkClicked(){
  std::cout<<"Return...\n";
  remove_modal_grab();

  if(timeGrab==0){
    timer->Start();
  }

  delete this;
}

void ScoreDialog::BeginnerChecked(){
  beginnerFlag=beginnerCheck->get_active();
  std::cout<<"Beginner"<<std::endl;
  if(beginnerFlag){
    std::cout<<"ACTIVE"<<std::endl;
  }
  else{
    std::cout<<"NO ACTIVE"<<std::endl;
  }
  ScoreDialog::ReadFile();
  //  ScoreDialog::SetNum();
}
void ScoreDialog::MediumChecked(){
  mediumFlag=mediumCheck->get_active();
  std::cout<<"Medium"<<std::endl;
  if(mediumFlag){
    std::cout<<"ACTIVE"<<std::endl;
  }
  else{
    std::cout<<"NO ACTIVE"<<std::endl;
  }
  ScoreDialog::ReadFile();
  //  ScoreDialog::SetNum();
}
void ScoreDialog::AdvanceChecked(){
  advanceFlag=advanceCheck->get_active();
  std::cout<<"Advance"<<std::endl;
  if(advanceFlag){
    std::cout<<"ACTIVE"<<std::endl;
  }
  else{
    std::cout<<"NO ACTIVE"<<std::endl;
  }
  ScoreDialog::ReadFile();
  //  ScoreDialog::SetNum();
}
void ScoreDialog::FreeChecked(){
  freeFlag=freeCheck->get_active();
  std::cout<<"Free"<<std::endl;
  if(freeFlag){
    std::cout<<"ACTIVE"<<std::endl;
  }
  else{
    std::cout<<"NO ACTIVE"<<std::endl;
  }
  ScoreDialog::ReadFile();
  //  ScoreDialog::SetNum();
}

void ScoreDialog::SelectMode(char *buffer0,int mode){
  if(mode==0){
    sprintf(buffer0,"unknown");
  }
  else if(mode==1){
    sprintf(buffer0,"Beginner");
  }
  else if(mode==2){
    sprintf(buffer0,"Medium");
  }
  else if(mode==3){
    sprintf(buffer0,"Advance");
  }
  else if(mode==4){
    sprintf(buffer0,"Free");
  }
}

void ScoreDialog::Create(int targetRow0){
  timeGrab=0;
  targetRow=targetRow0;

  timeGrab=timer->Stop();
  add_modal_grab();
  
    ReadFile();
    //  SetNum();

  scrWin->set_policy(Gtk::POLICY_NEVER,Gtk::POLICY_AUTOMATIC);
  scrWin->add(*listView);
  listView->columns_autosize();
  listView->set_size_request(550,250);
  vbox->pack_start(*scrWin);
  scrWin->show();
  listView->show();

  checkBox->attach(*beginnerCheck,0,1,0,1,Gtk::SHRINK,Gtk::SHRINK,0,0);
  checkBox->attach(*mediumCheck,1,2,0,1,Gtk::SHRINK,Gtk::SHRINK,0,0);
  checkBox->attach(*advanceCheck,2,3,0,1,Gtk::SHRINK,Gtk::SHRINK,0,0);
  checkBox->attach(*freeCheck,3,4,0,1,Gtk::SHRINK,Gtk::SHRINK,0,0);
  beginnerCheck->set_active(true);
  mediumCheck->set_active(true);
  advanceCheck->set_active(true);
  freeCheck->set_active(true);
  beginnerCheck->show();
  mediumCheck->show();
  advanceCheck->show();
  freeCheck->show();
  checkBox->show();
  vbox->pack_start(*checkBox);
  

  okButton->show();
  vbox->show();
  show();

  // listView->collapse_all();

  okButton->signal_clicked().connect
    (sigc::mem_fun(*this,&ScoreDialog::OkClicked));
  signal_delete_event().connect
    (sigc::mem_fun(*this,&ScoreDialog::SignalDelete));
  beginnerCheck->signal_toggled().connect
    (sigc::mem_fun(*this,&ScoreDialog::BeginnerChecked));
  mediumCheck->signal_toggled().connect
    (sigc::mem_fun(*this,&ScoreDialog::MediumChecked));
  advanceCheck->signal_toggled().connect
    (sigc::mem_fun(*this,&ScoreDialog::AdvanceChecked));
  freeCheck->signal_toggled().connect
    (sigc::mem_fun(*this,&ScoreDialog::FreeChecked));

}

void ScoreDialog::SetNum(){
  int i=1;
  
  
  type_children children =mList->children();
  Gtk::TreeModel::Row row;
  Gtk::TreeModel::Path path;
  type_children::iterator iter = children.begin();
  type_children::iterator tmp;
  while(iter!=children.end()){

    /*
    if(i>30){
      tmp=iter;
      tmp++;
      children.end()=mList->erase(iter);
    }
    */
  
    row=*iter;
    row[scoreCol->num]=i;
    
    
    i++;
    iter++;
  }
}

void ScoreDialog::ReadFile(){
  int mFlag=0,i=0;
  ScoreItem *node;
  ScoreItem *tmp;

  char buffer[255];

  mList->clear();

  type_children children = mList->children();
  Gtk::TreeModel::Row row;
  Gtk::TreeModel::Path path;
  type_children::iterator iter;
  i=1;
  node=scoreFile->firstList;
  while(node){
    tmp=node->next;
    
    if(i>30){
      break;
    }

    if(node->mode==1){
      if(beginnerFlag==1){
	mFlag=1;
      }
    }
    else if(node->mode==2){
      if(mediumFlag==1){
	mFlag=1;
      }
    }
    else if(node->mode==3){
      if(advanceFlag==1){
	mFlag=1;
      }
    }
    else if(node->mode==4){
      if(freeFlag==1){
	mFlag=1;
      }
    }
    
    if(mFlag==1){   
      row=*(mList->append());
      row[scoreCol->num]=i;
      row[scoreCol->score]=node->score;
      row[scoreCol->board]=node->col*node->row;
      sprintf(buffer,"%d (%dx%d)",
	      node->col*node->row,node->row,node->col);
      row[scoreCol->strBoard]=buffer;
      row[scoreCol->bomb]=node->bomb;
      row[scoreCol->sec]=node->sec;
      sprintf(buffer,"%d.%d sec",node->sec/10,node->sec%10);
      row[scoreCol->strSec]=buffer;
      row[scoreCol->rate]=(node->bomb*100)/(node->row*node->col);
      if(node->name.size()==0){
	row[scoreCol->strName]="NO NAME";
      }
      else{
	row[scoreCol->strName]=node->name;
      }
      ScoreDialog::SelectMode(buffer,node->mode);
      
      row[scoreCol->strMode]=buffer;
      //   std::cout<<node->score<<std::endl;
      if(targetRow==node->TIME){
	path=mList->get_path(row);
	listView->set_cursor(path);
	listView->scroll_to_row(path);
      }
      i++;   
    }

    mFlag=0;
    node=tmp;
  }
}
